# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import collections

# project
from .command import Command
from .logger import log


class Path(object):
    """
    Directory path helpers
    """
    @classmethod
    def sort_by_hierarchy(self, path_list):
        """
        Sort given list of path names by their hierachy in the tree

        :param list path_list: list of path names

        :return: sorted path_list
        :rtype: list
        """
        paths_at_depth = {}
        for path in path_list:
            path_elements = path.split('/')
            path_depth = len(path_elements)
            if path_depth not in paths_at_depth:
                paths_at_depth[path_depth] = []
            paths_at_depth[path_depth].append(path)
        ordered_paths_at_depth = collections.OrderedDict(
            sorted(paths_at_depth.items())
        )
        ordered_paths = []
        for path_depth in ordered_paths_at_depth:
            for path in ordered_paths_at_depth[path_depth]:
                ordered_paths.append(path)
        return ordered_paths

    @classmethod
    def create(self, path):
        """
        Create path and all sub directories to target

        :param string path: path name
        """
        Command.run(
            ['mkdir', '-p', path]
        )

    @classmethod
    def wipe(self, path):
        """
        Delete path and all contents

        :param string path: path name
        """
        Command.run(
            ['rm', '-r', '-f', path]
        )

    @classmethod
    def remove(self, path):
        """
        Delete empty path, causes an error if target is not empty

        :param string path: path name
        """
        Command.run(
            ['rmdir', path]
        )

    @classmethod
    def remove_hierarchy(self, path):
        """
        Recursively remove an empty path and its sub directories
        ignore non empty paths and leave them untouched

        :param string path: path name
        """
        Command.run(
            ['rmdir', '-p', '--ignore-fail-on-non-empty', path]
        )

    @classmethod
    def which(
        self, filename, alternative_lookup_paths=None,
        custom_env=None, access_mode=None
    ):
        """
        Lookup file name in PATH

        :param string filename: file base name
        :param list alternative_lookup_paths: list of additional lookup paths
        :param list custom_env: a custom os.environ
        :param int mode: one of the os access modes or a combination of
         them (os.R_OK, os.W_OK and os.X_OK). If the provided access mode
         does not match the file is considered not existing
        """
        lookup_paths = []
        multipart_message = [
            '"%s": ' % filename, 'exists: unknown', 'mode match: not checked'
        ]
        system_path = os.environ.get('PATH')
        if custom_env:
            system_path = custom_env.get('PATH')
        if system_path:
            lookup_paths = system_path.split(os.pathsep)
        if alternative_lookup_paths:
            lookup_paths += alternative_lookup_paths
        multipart_message[0] += 'in paths "%s"' % ':'.join(lookup_paths)
        for path in lookup_paths:
            location = os.path.join(path, filename)
            file_exists = os.path.exists(location)
            multipart_message[1] = 'exists: "%s"' % file_exists
            if access_mode and file_exists:
                mode_match = os.access(location, access_mode)
                multipart_message[2] = 'mode match: "%s"' % mode_match
                if mode_match:
                    return location
            elif file_exists:
                return location

        log.debug(' '.join(multipart_message))




from mock import patch

import mock
import os

from .test_helper import *

from kiwi.exceptions import (
    KiwiUriStyleUnknown,
    KiwiUriTypeUnknown
)

from kiwi.system.profile import Profile
from kiwi.xml_state import XMLState
from kiwi.xml_description import XMLDescription


class TestProfile(object):
    def setup(self):
        self.tmpfile = mock.Mock()
        self.tmpfile.name = 'tmpfile'
        description = XMLDescription('../data/example_dot_profile_config.xml')
        self.profile = Profile(
            XMLState(description.load())
        )

    @patch('kiwi.system.profile.NamedTemporaryFile')
    @patch('kiwi.path.Path.which')
    def test_create(self, mock_which, mock_temp):
        mock_which.return_value = 'cp'
        mock_temp.return_value = self.tmpfile
        result = self.profile.create()
        os.remove(self.tmpfile.name)
        assert self.profile.dot_profile == {
            'kiwi_Volume_1': 'usr_lib|size:1024|usr/lib',
            'kiwi_Volume_2': 'LVRoot|freespace:500|',
            'kiwi_Volume_3': 'etc_volume|freespace:30|etc',
            'kiwi_Volume_4': 'bin_volume|size:all|/usr/bin',
            'kiwi_Volume_5': 'usr_bin|size:all|usr/bin',
            'kiwi_bootkernel': None,
            'kiwi_bootloader': 'grub2',
            'kiwi_bootprofile': None,
            'kiwi_target_removable': None,
            'kiwi_boot_timeout': None,
            'kiwi_cmdline': 'splash',
            'kiwi_compressed': None,
            'kiwi_delete': '',
            'kiwi_devicepersistency': None,
            'kiwi_bootloader_console': None,
            'kiwi_displayname': 'LimeJeOS-openSUSE-13.2',
            'kiwi_drivers': '',
            'kiwi_firmware': 'efi',
            'kiwi_fsmountoptions': None,
            'kiwi_hwclock': 'utc',
            'kiwi_hybrid': True,
            'kiwi_hybridpersistent_filesystem': None,
            'kiwi_hybridpersistent': None,
            'kiwi_iname': 'LimeJeOS-openSUSE-13.2',
            'kiwi_installboot': None,
            'kiwi_iversion': '1.13.2',
            'kiwi_keytable': 'us.map.gz',
            'kiwi_language': 'en_US',
            'kiwi_loader_theme': 'openSUSE',
            'kiwi_lvm': 'true',
            'kiwi_lvmgroup': 'systemVG',
            'kiwi_oemataraid_scan': None,
            'kiwi_oembootwait': None,
            'kiwi_oemdevicefilter': None,
            'kiwi_oemkboot': None,
            'kiwi_oemmultipath_scan': None,
            'kiwi_oempartition_install': None,
            'kiwi_oemrebootinteractive': None,
            'kiwi_oemreboot': None,
            'kiwi_oemrecoveryID': None,
            'kiwi_oemrecoveryInPlace': None,
            'kiwi_oemrecovery': False,
            'kiwi_oemrecoveryPartSize': None,
            'kiwi_oemrootMB': 2048,
            'kiwi_oemshutdowninteractive': None,
            'kiwi_oemshutdown': None,
            'kiwi_oemsilentboot': None,
            'kiwi_oemsilentinstall': None,
            'kiwi_oemsilentverify': None,
            'kiwi_oemskipverify': None,
            'kiwi_oemswapMB': None,
            'kiwi_oemswap': 'true',
            'kiwi_oemtitle': None,
            'kiwi_oemunattended_id': None,
            'kiwi_oemunattended': None,
            'kiwi_oemvmcp_parmfile': None,
            'kiwi_profiles': '',
            'kiwi_ramonly': None,
            'kiwi_initrd_system': None,
            'kiwi_btrfs_root_is_snapshot': None,
            'kiwi_gpt_hybrid_mbr': None,
            'kiwi_showlicense': None,
            'kiwi_splash_theme': 'openSUSE',
            'kiwi_strip_delete': '',
            'kiwi_strip_libs': '',
            'kiwi_strip_tools': '',
            'kiwi_target_blocksize': None,
            'kiwi_timezone': 'Europe/Berlin',
            'kiwi_type': 'oem',
            'kiwi_vga': None,
            'kiwi_wwid_wait_timeout': None,
            'kiwi_xendomain': None
        }
        assert result == [
            "kiwi_Volume_1='usr_lib|size:1024|usr/lib'",
            "kiwi_Volume_2='LVRoot|freespace:500|'",
            "kiwi_Volume_3='etc_volume|freespace:30|etc'",
            "kiwi_Volume_4='bin_volume|size:all|/usr/bin'",
            "kiwi_Volume_5='usr_bin|size:all|usr/bin'",
            "kiwi_bootloader='grub2'",
            "kiwi_cmdline='splash'",
            "kiwi_displayname='LimeJeOS-openSUSE-13.2'",
            "kiwi_firmware='efi'",
            "kiwi_hwclock='utc'",
            "kiwi_hybrid='true'",
            "kiwi_iname='LimeJeOS-openSUSE-13.2'",
            "kiwi_iversion='1.13.2'",
            "kiwi_keytable='us.map.gz'",
            "kiwi_language='en_US'",
            "kiwi_loader_theme='openSUSE'",
            "kiwi_lvm='true'",
            "kiwi_lvmgroup='systemVG'",
            "kiwi_oemrootMB='2048'",
            "kiwi_oemswap='true'",
            "kiwi_splash_theme='openSUSE'",
            "kiwi_timezone='Europe/Berlin'",
            "kiwi_type='oem'"
        ]

    @patch('kiwi.system.profile.NamedTemporaryFile')
    @patch('kiwi.path.Path.which')
    def test_create_cpio(self, mock_which, mock_temp):
        mock_which.return_value = 'cp'
        mock_temp.return_value = self.tmpfile
        description = XMLDescription('../data/example_dot_profile_config.xml')
        profile = Profile(
            XMLState(description.load(), None, 'cpio')
        )
        profile.create()
        os.remove(self.tmpfile.name)
        assert profile.dot_profile['kiwi_cpio_name'] == 'LimeJeOS-openSUSE-13.2'

    def test_add(self):
        self.profile.add('foo', 'bar')
        assert self.profile.dot_profile['foo'] == 'bar'

    def test_delete(self):
        self.profile.add('foo', 'bar')
        self.profile.delete('foo')
        assert 'foo' not in self.profile.dot_profile



import sys
import mock

from mock import patch

import logging
from .test_helper import *
import inspect

from kiwi.tasks.base import CliTask
from kiwi.exceptions import *

import kiwi.xml_parse


class TestCliTask(object):
    @patch('os.path.isfile')
    @patch('configparser.ConfigParser.has_section')
    @patch('kiwi.logger.log.setLogLevel')
    @patch('kiwi.logger.log.set_logfile')
    @patch('kiwi.logger.log.set_color_format')
    @patch('kiwi.cli.Cli.show_and_exit_on_help_request')
    def setup(
        self, mock_help, mock_color, mock_setlog, mock_setlevel,
        mock_section, mock_isfile
    ):
        sys.argv = [
            sys.argv[0],
            '--debug',
            '--logfile', 'log',
            '--color-output',
            '--profile', 'vmxFlavour',
            'system',
            'prepare',
            '--description', 'description',
            '--root', 'directory'
        ]
        self.task = CliTask()

        mock_help.assert_called_once_with()
        mock_setlevel.assert_called_once_with(logging.DEBUG)
        mock_setlog.assert_called_once_with('log')
        mock_color.assert_called_once_with()

    def test_quadruple_token(self):
        assert self.task.quadruple_token('a,b') == ['a', 'b', None, None]

    @patch('kiwi.tasks.base.RuntimeChecker')
    def test_load_xml_description(self, mock_runtime_checker):
        self.task.load_xml_description('../data/description')
        mock_runtime_checker.assert_called_once_with(self.task.xml_state)
        assert self.task.config_file == '../data/description/config.xml'
        assert isinstance(self.task.xml_data, kiwi.xml_parse.image)
        assert self.task.xml_state.profiles == ['vmxFlavour']

    def test_load_xml_description_buildservice(self):
        self.task.load_xml_description('../data/description.buildservice')
        assert self.task.config_file == \
            '../data/description.buildservice/appliance.kiwi'

    @raises(KiwiConfigFileNotFound)
    def test_load_xml_description_raises(self):
        self.task.load_xml_description('foo')




from mock import patch

import mock

from .test_helper import *

from kiwi.defaults import Defaults


class TestDefaults(object):
    def setup(self):
        self.defaults = Defaults()

    def test_get(self):
        assert self.defaults.get('kiwi_align') == 1048576
        assert self.defaults.get('kiwi_startsector') == 2048
        assert self.defaults.get('kiwi_sectorsize') == 512
        assert self.defaults.get('kiwi_inode_size') == 256
        assert self.defaults.get('kiwi_inode_ratio') == 16384
        assert self.defaults.get('kiwi_min_inodes') == 20000
        assert self.defaults.get('kiwi_revision')

    def test_to_profile(self):
        profile = mock.MagicMock()
        self.defaults.to_profile(profile)
        profile.add.assert_any_call('kiwi_align', 1048576)
        profile.add.assert_any_call('kiwi_startsector', 2048)
        profile.add.assert_any_call('kiwi_sectorsize', 512)
        profile.add.assert_any_call(
            'kiwi_revision', self.defaults.get('kiwi_revision')
        )

    def test_get_preparer(self):
        assert Defaults.get_preparer() == 'KIWI - http://suse.github.com/kiwi'

    def test_get_publisher(self):
        assert Defaults.get_publisher() == 'SUSE LINUX GmbH'
